import { useState, useEffect, useMemo } from "react";
import { Box, Container, Typography, useMediaQuery } from "@mui/material";
import { useConfig, useBalance } from "wagmi";
import { Helmet } from "react-helmet";
import ReactGA from "react-ga4";

import PageTitle from "../../components/PageTitle/PageTitle";
import Paper from "../../components/Paper/Paper";
import SwapCard from "../../components/Swap/SwapCard";
import TokenStack from "../../components/TokenStack/TokenStack";
import { PrimaryButton } from "../../components/Button";

import { DAI_ADDRESSES } from "../../constants/addresses";
import { DecimalBigNumber } from "../../helpers/DecimalBigNumber";
import { formatCurrency, formatNumber } from "../../helpers";

import { useBalance as useTokenBalance, useConversionRate, mintDai } from "../../hooks/tokens";

const Faucet = ({ chainId, address, config, connect }) => {
    const isSmallScreen = useMediaQuery("(max-width: 650px)");
    const isSemiSmallScreen = useMediaQuery("(max-width: 480px)");
    const isVerySmallScreen = useMediaQuery("(max-width: 379px)");

    const daiConversionRate = useConversionRate(chainId, "GDAI");
    const { balance: daiBalance, refetch: daiBalanceRefetch } = useTokenBalance(chainId, "GDAI", address);
    const { data: nativeBalance, refetch: balanceRefetch } = useBalance({ address });

    const [isPending, setIsPending] = useState(false);
    const [balance, setBalance] = useState(new DecimalBigNumber(0, 0));
    const [amount, setAmount] = useState("");
    const [faucetToken, setFaucetToken] = useState({
        name: "",
        address: "",
    });
    const [scanInfo, setScanInfo] = useState({
        name: "",
        url: "",
    });

    useEffect(() => {
        ReactGA.send({ hitType: "pageview", page: "/faucet" });
    }, [])

    useEffect(() => {
        const value = nativeBalance ? nativeBalance.value : 0n;
        const decimals = nativeBalance ? nativeBalance.decimals : 18;
        setBalance(new DecimalBigNumber(value, decimals));
    }, [nativeBalance]);

    useEffect(() => {
        let scanName = "";
        let scanUrl = "";

        const tokenName = "gDAI";
        const tokenAddress = DAI_ADDRESSES[chainId];

        const client = config?.getClient();
        scanName = client?.chain?.blockExplorers?.default?.name;
        scanUrl = client?.chain?.blockExplorers?.default?.url;

        setFaucetToken({
            name: tokenName,
            address: tokenAddress,
        });

        setScanInfo({
            name: scanName,
            url: scanUrl,
        })
    }, [chainId]);

    const preparedAmount = useMemo(() => {
        if (address === "") new DecimalBigNumber("0", 0);
        return new DecimalBigNumber(amount, 18);
    }, [amount, balance])

    const estimatedAmount = useMemo(() => {
        const rate = new DecimalBigNumber(daiConversionRate.toString(), 0);
        const value = new DecimalBigNumber(amount, 18);
        return value.mul(rate);
    }, [amount, daiConversionRate])

    const mintOrConnect = async () => {
        if (address === "") {
            connect();
        } else {
            setIsPending(true);
            await mintDai(chainId, address, preparedAmount._value.toString());

            await balanceRefetch();
            await daiBalanceRefetch();
            setAmount("");
            setIsPending(false);
        }
    }

    return (
        <Box height="calc(100vh - 43px)">
            <Helmet>
                <title>ghostFaucet | gDAI Faucet</title>
                <meta name="description" content="ghostFaucet is a No KYC gDAI Faucet powered by GHOST. Get gDAI sent directly to your wallet." />
                <meta name="keywords" content="ghostFaucet, web3 faucet, gDAI, ethereum, sepolia, polygon, bnb, bsc, AVAX" />

                <meta property="og:image" content="https://ghostchain.io/wp-content/uploads/2025/03/ghostFaucet-Featured_Image.png" />
                <meta property="og:title" content="ghostDAO | The DeFi 2.0 cross-chain reserve currency" />
                <meta property="og:image:type" content="image/png" />
                <meta property="og:image:width" content="1200" />
                <meta property="og:image:height" content="630" />
                <meta property="og:type" content="website" />
                <meta property="og:description" content="ghostFaucet is a No KYC gDAI Faucet powered by GHOST. Get gDAI sent directly to your wallet." />

                <meta name="twitter:card" content="summary" />
                <meta name="twitter:site" content="@realGhostChain" />
                <meta name="twitter:title" content="ghostDAO | The DeFi 2.0 cross-chain reserve currency" />
                <meta name="twitter:description" content="ghostFaucet is a No KYC gDAI Faucet powered by GHOST. Get gDAI sent directly to your wallet." />
                <meta name="twitter:image" content="https://ghostchain.io/wp-content/uploads/2025/03/ghostFaucet-Featured_Image.png" />
            </Helmet>

            <PageTitle name={"gDAI Faucet"} subtitle="Swap Sepolia ETH for gDAI." />
            <Container
                style={{
                    paddingLeft: isSmallScreen || isVerySmallScreen ? "0" : "3.3rem",
                    paddingRight: isSmallScreen || isVerySmallScreen ? "0" : "3.3rem",
                    display: "flex",
                    justifyContent: "center",
                    alignItems: "center",
                    height: "calc(100vh - 153px)"
                }}
            >
                <Box width="100%" maxWidth="476px" display="flex" alignItems="center" justifyContent="center" flexDirection="column">
                    <Paper
                        headerContent={
                            <Box alignItems="center" justifyContent="space-between" display="flex" width="100%">
                                <Typography variant="h4">Get {faucetToken.name}</Typography>
                                {!isSemiSmallScreen && <PrimaryButton
                                    variant="text"
                                    href={`${scanInfo.url}/token/${faucetToken.address}`}
                                >
                                    Check on {scanInfo.name}
                                </PrimaryButton>}
                            </Box>
                        }
                        enableBackground
                        fullWidth
                    >
                        <Box>
                            <SwapCard
                                id={`faucet-sepolia-eth`}
                                inputWidth={isVerySmallScreen ? "100px" : isSemiSmallScreen ? "180px" : "250px"}
                                tokenName={"ETH"}
                                token={<TokenStack tokens={["ETH"]} sx={{ fontSize: "21px" }} />}
                                info={`${isSemiSmallScreen ? "" : "Balance: "}${formatCurrency(balance.toString(), 4, "ETH")}`}
                                value={amount}
                                onChange={event => setAmount(event.currentTarget.value)}
                                inputProps={{ "data-testid": "fromInput" }}
                            />
                        <Box
                            mb="20px"
                            mt="20px"
                            flexDirection="column"
                            display="flex"
                            justifyContent="space-between"
                        >
                                <Box maxWidth="416px" display="flex" justifyContent={isVerySmallScreen ? "end" : "space-between"}>
                                    {!isVerySmallScreen && <Typography fontSize="12px" lineHeight="15px">ETH multiplier:</Typography>}
                                    <Typography fontSize="12px" lineHeight="15px">{formatNumber(daiConversionRate, 2)}</Typography>
                                </Box>
                                <Box maxWidth="416px" display="flex" justifyContent={isVerySmallScreen ? "end" : "space-between"}>
                                    {!isVerySmallScreen && <Typography fontSize="12px" lineHeight="15px">You will get:</Typography>}
                                    <Typography fontSize="12px" lineHeight="15px">{formatCurrency(estimatedAmount, 5, faucetToken.name)}</Typography>
                                </Box>
                                <Box display="flex" justifyContent={isVerySmallScreen ? "end" : "space-between"}>
                                    {!isVerySmallScreen && <Typography fontSize="12px" lineHeight="15px">Your {faucetToken.name} balance:</Typography>}
                                    <Typography fontSize="12px" lineHeight="15px">{formatCurrency(daiBalance, 5, faucetToken.name)}</Typography>
                                </Box>
                            </Box>
                            <PrimaryButton
                                fullWidth
                                disabled={
                                    address !== "" && (
                                        preparedAmount?.eq(new DecimalBigNumber(0, 18)) ||
                                        balance?.lt(preparedAmount) ||
                                        isPending
                                    )
                                }
                                loading={isPending}
                                onClick={() => mintOrConnect()}
                            >
                                {address === "" ?
                                    "Connect"
                                    :
                                    "Mint"
                                }
                            </PrimaryButton>
                        </Box>
                    </Paper>
                </Box>
            </Container>
        </Box>
    )
}

export default Faucet;
